@@ -3,6 +3,7 @@
 (function(exports) {
 
 const rules = require('./rules');
+const util = require("./util");
 
 function initialize() {
   return new Promise(resolve => {
@@ -96,12 +97,28 @@ async function performMigrations() {
     migration_version = 2;
     await set_promise('migration_version', migration_version);
   }
+
+  if (migration_version <= 2) {
+    await get_promise('disabledList', [])
+      .then(disabledList => {
+        disabledList = disabledList.map(item => {
+          return util.getNormalisedHostname(item);
+        })
+        return disabledList;
+      })
+      .then(disabledList => {
+        return set_promise('disabledList', disabledList);
+      })
+
+    migration_version = 3;
+    await set_promise('migration_version', migration_version);
+  }
 }
 
 const local = {
-  get: chrome.storage.local.get,
-  set: chrome.storage.local.set,
-  remove: chrome.storage.local.remove,
+  get: (...args) => chrome.storage.local.get(...args),
+  set: (...args) => chrome.storage.local.set(...args),
+  remove: (...args) => chrome.storage.local.remove(...args),
   get_promise: local_get_promise,
   set_promise: local_set_promise
 };
